package es.uva.tfg.geolocalizationgame.conf;

import java.util.Properties;

import javax.persistence.EntityManagerFactory;
import javax.sql.DataSource;
import javax.validation.Validator;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.jdbc.datasource.DriverManagerDataSource;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.orm.jpa.vendor.HibernateJpaVendorAdapter;
import org.springframework.validation.beanvalidation.LocalValidatorFactoryBean;
import org.springframework.validation.beanvalidation.MethodValidationPostProcessor;
import org.springframework.web.servlet.view.InternalResourceViewResolver;

/**
 * Implementa la configuracion de la aplicacion.
 * Configuracion del enrutamiento, configuracion de la base de datos
 * y configuracion de la validacion de entidades de la aplicacion
 * 
 * @author Cristina Gómez Escribano
 * @version 1.1
 * @since 04/02/2018
 *
 */
@Configuration
@ComponentScan(basePackages = { "es.uva.tfg.geolocalizationgame" })
@EnableJpaRepositories(basePackages = { "es.uva.tfg.geolocalizationgame.persistence" })
public class GeolocalizationGameConfiguration {

	/**
	 * Bean de configuracion con los datos de conexion de la base de datos
	 * 
	 * @param env
	 * @return DriverManagerDataSource
	 */
	@Bean
	DataSource dataSource(Environment env) {
		DriverManagerDataSource dmds = new DriverManagerDataSource();
		dmds.setDriverClassName("com.mysql.jdbc.Driver");
		dmds.setUrl("jdbc:mysql://localhost:3306/dbgeolocalizationgame?useSSL=false");
		dmds.setUsername("root");
		dmds.setPassword("1234");
		return dmds;
	}

	/**
	 * Bean del entity manager con las propiedades de jpa de la base de datos
	 * 
	 * @param dataSource
	 * @param env
	 * @return Entity manager
	 */
	@Bean
	LocalContainerEntityManagerFactoryBean entityManagerFactory(DataSource dataSource, Environment env) {
		LocalContainerEntityManagerFactoryBean entityManagerFactoryBean = new LocalContainerEntityManagerFactoryBean();
		entityManagerFactoryBean.setDataSource(dataSource);
		entityManagerFactoryBean.setJpaVendorAdapter(new HibernateJpaVendorAdapter());
		entityManagerFactoryBean.setPackagesToScan("es.uva.tfg.geolocalizationgame.resources");

		Properties jpaProperties = new Properties();
		jpaProperties.put("hibernate.dialect", "org.hibernate.dialect.MySQL5Dialect");
		jpaProperties.put("hibernate.hbm2ddl.auto", "update");
		entityManagerFactoryBean.setJpaProperties(jpaProperties);

		return entityManagerFactoryBean;
	}

	/**
	 * Creacion del transaction manager de jpa para realizar las operaciones en
	 * la base de datos
	 * 
	 * @param entityManagerFactory
	 * @return transaction manager
	 */
	@Bean
	JpaTransactionManager transactionManager(EntityManagerFactory entityManagerFactory) {
		JpaTransactionManager transactionManager = new JpaTransactionManager();
		transactionManager.setEntityManagerFactory(entityManagerFactory);
		return transactionManager;
	}

	/**
	 * Configuracion para validar los campos de las entidades cuando se crean
	 * 
	 * @return MethodValidationPostProcessor
	 */
	@Bean
	public MethodValidationPostProcessor methodValidationPostProcessor() {

		MethodValidationPostProcessor processor = new MethodValidationPostProcessor();
		processor.setValidator(validator());
		return processor;
	}

	/**
	 * Configuracion para utilizar el validated de las entidades
	 * 
	 * @return Validator
	 */
	@Bean
	public Validator validator() {
		return new LocalValidatorFactoryBean();
	}

	/**
	 * Configuracion para arrancar los html desde el servidor, con la ruta
	 * indicada
	 * 
	 * @return InternalResourceViewResolver
	 */
	@Bean
	public InternalResourceViewResolver viewResolver() {
		InternalResourceViewResolver resolver = new InternalResourceViewResolver();
		resolver.setPrefix("/");
		resolver.setSuffix(".html");
		return resolver;
	}
}
