package es.uva.tfg.geolocalizationgame.controllers;

import java.util.List;

import javax.validation.ConstraintViolationException;

import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.fasterxml.jackson.annotation.JsonView;

import es.uva.tfg.geolocalizationgame.exceptions.ComentarioNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.JuegoNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.MalformedRequestException;
import es.uva.tfg.geolocalizationgame.exceptions.UserNotCreatorOfComentarioException;
import es.uva.tfg.geolocalizationgame.exceptions.UserNotFoundException;
import es.uva.tfg.geolocalizationgame.resources.Comentario;
import es.uva.tfg.geolocalizationgame.resources.DatosUser;
import es.uva.tfg.geolocalizationgame.resources.ErrorMessage;
import es.uva.tfg.geolocalizationgame.resources.Juego;
import es.uva.tfg.geolocalizationgame.resources.Usuario;
import es.uva.tfg.geolocalizationgame.services.ComentarioService;
import es.uva.tfg.geolocalizationgame.services.JuegoService;
import es.uva.tfg.geolocalizationgame.services.UserService;

/**
 * Rest controller que recibe las peticiones de la entidad Comentario.
 * 
 * @author Cristina Gomez Escribano
 * @version 3.0
 * @since 9/10/2018
 *
 */
@RestController
@RequestMapping("juegos/{idJuego}/comentarios")
public class ComentarioRestController {

	@Autowired
	private ComentarioService comentarioService;
	
	@Autowired
	private UserService userService;
	
	@Autowired
	private JuegoService juegoService;
	
	/**
	 * Recibe la peticion para crear un nuevo comentario.
	 * @param idJuego
	 * @param comentario
	 * @return
	 * @throws MalformedRequestException
	 * @throws UserNotFoundException
	 * @throws JuegoNotFoundException
	 */
	@RequestMapping(method = RequestMethod.POST, consumes=MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<Void> postComentario(@PathVariable Integer idJuego, @RequestBody String comentario) throws MalformedRequestException, UserNotFoundException, JuegoNotFoundException {
		try {
			JSONObject json = new JSONObject(comentario);
			String descripcion = json.getString("descripcion");
			String creador = json.getString("creador");
			Usuario usuario=userService.obtenerUsuarioByLogin(creador);
			if(usuario==null) {
				throw new UserNotFoundException();
			}
			Juego juego = juegoService.obtenerJuegoById(idJuego);
			if(juego==null) {
				throw new JuegoNotFoundException();
			}
			Comentario comentarioNew = comentarioService.crearComentario(descripcion, usuario, juego);
			comentarioService.saveComentario(comentarioNew);
			return new ResponseEntity<>(HttpStatus.CREATED);
		
		} catch (JSONException e) {
			
			throw new MalformedRequestException("Json inválido");
			
		} catch(ConstraintViolationException ex) {
			
			throw new MalformedRequestException(ErrorMessage.getErrores(ex.getConstraintViolations()));
			
		}
	}
	
	/**
	 * Peticion para obtener todos los comentarios de un juego concreto.
	 * 
	 * @param idJuego
	 * @return
	 * @throws JuegoNotFoundException
	 */
	@JsonView(DatosUser.GameView.class)
	@RequestMapping(method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<List<Comentario>> getComentarios(@PathVariable Integer idJuego) throws JuegoNotFoundException {
		Juego juego= juegoService.obtenerJuegoById(idJuego);
		if(juego==null) {
			throw new JuegoNotFoundException();
		}
		return new ResponseEntity<List<Comentario>>(comentarioService.obtenerComentariosByJuego(juego), HttpStatus.OK);
	}
	
	/**
	 * Peticion para actualizar un comentario de un juego concreto.
	 * 
	 * @param idJuego
	 * @param idComentario
	 * @param comentarioModificado
	 * @return
	 * @throws UserNotFoundException
	 * @throws JuegoNotFoundException
	 * @throws ComentarioNotFoundException
	 * @throws UserNotCreatorOfComentarioException
	 * @throws MalformedRequestException
	 */
	@RequestMapping(value="{idComentario}", method = RequestMethod.PUT, produces = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<Comentario> putComentario(@PathVariable Integer idJuego, @PathVariable Integer idComentario, @RequestBody String comentarioModificado) throws UserNotFoundException, JuegoNotFoundException, ComentarioNotFoundException, UserNotCreatorOfComentarioException, MalformedRequestException {
		try {
			JSONObject json = new JSONObject(comentarioModificado);
			String descripcion = json.getString("descripcion");
			Authentication auth = SecurityContextHolder.getContext().getAuthentication();
			Usuario userAuth=userService.obtenerUsuarioByLogin(auth.getName());
			if(userAuth==null) {
				throw new UserNotFoundException();
			}
			Juego juego = juegoService.obtenerJuegoById(idJuego);
			if(juego==null) {
				throw new JuegoNotFoundException();
			}
			Comentario comentario=comentarioService.obtenerComentarioById(idComentario);
			if(comentario==null) {
				throw new ComentarioNotFoundException();
			}
			if(!comentario.getCreador().equals(userAuth)) {//comprobar si es el creador del comentario
				throw new UserNotCreatorOfComentarioException();
			}
			if(!comentario.getJuego().equals(juego)) {//comprobar que sea un comentario del juego seleccionado
				throw new ComentarioNotFoundException();
			}
			comentarioService.updateComentario(comentario, descripcion);
			comentarioService.saveComentario(comentario);
			return new ResponseEntity<>(HttpStatus.OK);
		
		}catch(ConstraintViolationException ex) {
			throw new MalformedRequestException(ErrorMessage.getErrores(ex.getConstraintViolations()));
		} catch (JSONException e) {
			throw new MalformedRequestException("Json inválido");
		}
	}
	
	/**
	 * Recibe la petición para eliminar un comentario de un juego y un usuario.
	 * 
	 * @param idJuego
	 * @param idComentario
	 * @return
	 * @throws JuegoNotFoundException
	 * @throws ComentarioNotFoundException
	 * @throws UserNotFoundException
	 * @throws UserNotCreatorOfComentarioException
	 */
	@RequestMapping(value="{idComentario}", method = RequestMethod.DELETE)
	public ResponseEntity<Void> deleteComentario(@PathVariable Integer idJuego, @PathVariable Integer idComentario) throws JuegoNotFoundException, ComentarioNotFoundException, UserNotFoundException, UserNotCreatorOfComentarioException {
		Authentication auth = SecurityContextHolder.getContext().getAuthentication();
		Usuario userAuth=userService.obtenerUsuarioByLogin(auth.getName());
		if(userAuth==null) {
			throw new UserNotFoundException();
		}
		Juego juego = juegoService.obtenerJuegoById(idJuego);
		if(juego==null) {
			throw new JuegoNotFoundException();
		}
		Comentario comentario=comentarioService.obtenerComentarioById(idComentario);
		if(comentario==null) {
			throw new ComentarioNotFoundException();
		}
		if(!comentario.getCreador().equals(userAuth)) {//comprobar si es el creador del comentario
			throw new UserNotCreatorOfComentarioException();
		}
		if(!comentario.getJuego().equals(juego)) {//comprobar que sea un comentario del juego seleccionado
			throw new ComentarioNotFoundException();
		}
		comentarioService.deleteComentario(comentario);
		return new ResponseEntity<Void>(HttpStatus.OK);
	}
}
