package es.uva.tfg.geolocalizationgame.controllers;

import java.util.ArrayList;
import java.util.List;

import javax.validation.ConstraintViolationException;

import org.json.JSONObject;
import org.json.JSONArray;
import org.json.JSONException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.fasterxml.jackson.annotation.JsonView;

import es.uva.tfg.geolocalizationgame.exceptions.JuegoNotVisibleForUserException;
import es.uva.tfg.geolocalizationgame.exceptions.JuegoNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.MalformedRequestException;
import es.uva.tfg.geolocalizationgame.exceptions.TipoPuntoNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.TipoVisibilidadNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.UserNotCreatorOfJuegoException;
import es.uva.tfg.geolocalizationgame.exceptions.UserNotFoundException;
import es.uva.tfg.geolocalizationgame.resources.DatosUser;
import es.uva.tfg.geolocalizationgame.resources.ErrorMessage;
import es.uva.tfg.geolocalizationgame.resources.Juego;
import es.uva.tfg.geolocalizationgame.resources.PuntoGeografico;
import es.uva.tfg.geolocalizationgame.resources.Usuario;
import es.uva.tfg.geolocalizationgame.services.JuegoService;
import es.uva.tfg.geolocalizationgame.services.PuntoGeograficoService;
import es.uva.tfg.geolocalizationgame.services.UserService;

@RestController
@RequestMapping
public class JuegoRestController {

	@Autowired
	private JuegoService juegoService;
	
	@Autowired
	private UserService userService;
	
	@Autowired
	private PuntoGeograficoService puntoService;
	
	/**
	 * Recibe la peticion para crear un nuevo juego.
	 * 
	 * @param login
	 * @param juego
	 * @return ResponseEntity<Void>
	 * @throws UserNotFoundException 
	 * @throws TipoPuntoNotFoundException 
	 * @throws TipoVisibilidadNotFoundException 
	 * @throws MalformedRequestException 
	 */
	@RequestMapping(value="/usuarios/{login}/juegos", method = RequestMethod.POST, consumes=MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<Void> postJuego(@PathVariable String login, @RequestBody String juego) throws UserNotFoundException, TipoPuntoNotFoundException, TipoVisibilidadNotFoundException, MalformedRequestException {
		Usuario usuario = userService.obtenerUsuarioByLogin(login);
		if(usuario==null) {
			throw new UserNotFoundException();
		}
		List<PuntoGeografico> listaPuntos=new ArrayList<>();
		try {
			JSONObject json = new JSONObject(juego);
			String nombreJuego=json.getString("nombre");
			Juego juegoCreado=juegoService.crearJuego(usuario, nombreJuego);
			JSONArray puntos=json.getJSONArray("puntos");
			for(int i=0; i<puntos.length(); i++) {
				JSONObject punto= new JSONObject(puntos.get(i).toString());
				listaPuntos.add(puntoService.crearPunto(punto, juegoCreado));//add nuevo punto a la lista
			}
			juegoCreado.setPuntos(listaPuntos);//se guarda la lista de puntos
			juegoService.saveJuego(juegoCreado);
			return new ResponseEntity<Void>(HttpStatus.CREATED);
			
		}catch(JSONException e) {
			throw new MalformedRequestException("No se ha podido convertir el json");
		} catch(ConstraintViolationException ex) {
			throw new MalformedRequestException(ErrorMessage.getErrores(ex.getConstraintViolations()));
		}
	}
	
	/**
	 * Recibe la peticiÃ³n para eliminar un juego creado por un usuario.
	 * 
	 * @param login
	 * @param idJuego
	 * @return
	 * @throws UserNotFoundException
	 * @throws JuegoNotFoundException
	 * @throws UserNotCreatorOfJuegoException
	 */
	@RequestMapping(value="/usuarios/{login}/juegos/{idJuego}", method = RequestMethod.DELETE)
	public ResponseEntity<Void> deleteJuego(@PathVariable String login, @PathVariable Integer idJuego) throws UserNotFoundException, JuegoNotFoundException, UserNotCreatorOfJuegoException {
		Usuario usuario=userService.obtenerUsuarioByLogin(login);
		Juego juego=juegoService.obtenerJuegoById(idJuego);
		if(usuario==null) {
			throw new UserNotFoundException();
		}else if(juego==null) {
			throw new JuegoNotFoundException();
		}else if(!juego.getCreador().equals(usuario)) {//comprueba que es el creador del juego
			throw new UserNotCreatorOfJuegoException();
		}else {
			juegoService.eliminarJuego(juego);
			return new ResponseEntity<Void>(HttpStatus.OK);
		}
	}
	
	/**
	 * Recibe la peticion get para obtener los juegos de un usuario por login 
	 * @param login
	 * @return List<Juego>
	 * @throws UserNotFoundException
	 */
	@JsonView(DatosUser.PrivateProfile.class)
	@RequestMapping(value="/usuarios/{login}/juegos", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<List<Juego>> getJuegosByLogin(@PathVariable String login) throws UserNotFoundException {
		Usuario usuario=userService.obtenerUsuarioByLogin(login);
		if(usuario==null) {
			throw new UserNotFoundException();
		}else {
			return new ResponseEntity<>(juegoService.obtenerJuegosByLogin(usuario), HttpStatus.OK);
		} 
	}
	
	/**
	 * Recibe la peticiÃ³n get para obtener la lista de juegos. Puede incluir un
	 * parametro para buscar juegos por texto.
	 * 
	 * @param text
	 *            (no required)
	 * @return ResponseEntity<List<Juego>>
	 */
	@JsonView(DatosUser.PrivateProfile.class)
	@RequestMapping(value="/juegos", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<List<Juego>> getJuegos(@RequestParam(required = false) String text){
		List<Juego> juegos= juegoService.obtenerJuegosPublicosByText(text);
		return new ResponseEntity<>(juegos, HttpStatus.OK);
	}
	
	/**
	 * Recibe la peticiÃ³n get para obtener un juego por id.
	 * 
	 * @return ResponseEntity<Juego>
	 * @throws JuegoNotFoundException 
	 * @throws JuegoNotVisibleForUserException 
	 * @throws UserNotFoundException 
	 */
	@JsonView(DatosUser.GameView.class)
	@RequestMapping(value="/juegos/{idJuego}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<Juego> getJuegoById(@PathVariable Integer idJuego) throws JuegoNotFoundException, JuegoNotVisibleForUserException, UserNotFoundException{
		Authentication auth = SecurityContextHolder.getContext().getAuthentication();
		Usuario userAuth=userService.obtenerUsuarioByLogin(auth.getName());
		if(userAuth==null) {
			throw new UserNotFoundException();
		}
		Juego juego= juegoService.obtenerJuegoById(idJuego);
		if(juego==null) {
			throw new JuegoNotFoundException();
		}else if(!juegoService.compruebaJuegoPermitido(userAuth, juego)) {//devuelvo excepcion no permitido
				throw new JuegoNotVisibleForUserException();
		}
		return new ResponseEntity<Juego>(juego, HttpStatus.OK);
	}
	
	/**
	 * Recibe la peticion para actualizar un juego.
	 * 
	 * @param login
	 * @param idJuego
	 * @param nombre
	 * @param visibilidad
	 * @return
	 * @throws UserNotFoundException
	 * @throws JuegoNotFoundException
	 * @throws UserNotCreatorOfJuegoException
	 * @throws MalformedRequestException
	 * @throws TipoVisibilidadNotFoundException
	 */
	@JsonView(DatosUser.PrivateProfile.class)
	@RequestMapping(value="/usuarios/{login}/juegos/{idJuego}", method = RequestMethod.PUT, produces = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<Juego> putJuego(@PathVariable String login, @PathVariable Integer idJuego, @RequestParam(required = false) String nombre, @RequestParam(required = false) Integer visibilidad) throws UserNotFoundException, JuegoNotFoundException, UserNotCreatorOfJuegoException, MalformedRequestException, TipoVisibilidadNotFoundException{
		try {
			if(nombre==null && visibilidad==null || nombre!=null && visibilidad!=null) {//request invalida
				throw new MalformedRequestException("Los datos enviados no son validos.");
			}
			else {
				Usuario usuario=userService.obtenerUsuarioByLogin(login);
				if(usuario==null) {
					throw new UserNotFoundException();
				}
				Juego juego=juegoService.obtenerJuegoById(idJuego);
				if(juego==null) {
					throw new JuegoNotFoundException();
				}
				if(!juego.getCreador().equals(usuario)) {//no es el creador, no se puede cambiar
					throw new UserNotCreatorOfJuegoException();
				}else {//es el credor, del juego
					if(visibilidad!=null) {//se va a actualizar la visibilidad
						if(!juegoService.actualizarVisibilidad(juego, visibilidad)) {
							throw new TipoVisibilidadNotFoundException();
						}
					}else {//se va a actualizar el nombre
						juegoService.actualizarNombre(juego, nombre);
					}
					if(juegoService.updateJuego(juego)==null) {
						throw new MalformedRequestException();
					}
				}
				return new ResponseEntity<>(juego, HttpStatus.ACCEPTED);
			}
		}
		catch (ConstraintViolationException ex) {
			throw new MalformedRequestException(ErrorMessage.getErrores(ex.getConstraintViolations()));
		}
	}
}

