package es.uva.tfg.geolocalizationgame.controllers;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.fasterxml.jackson.annotation.JsonView;

import es.uva.tfg.geolocalizationgame.exceptions.LogroNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.MalformedRequestException;
import es.uva.tfg.geolocalizationgame.exceptions.TipoLogroNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.TipoVisibilidadNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.UserNotFoundException;
import es.uva.tfg.geolocalizationgame.resources.DatosUser;
import es.uva.tfg.geolocalizationgame.resources.Logro;
import es.uva.tfg.geolocalizationgame.resources.TipoLogro;
import es.uva.tfg.geolocalizationgame.resources.Usuario;
import es.uva.tfg.geolocalizationgame.services.LogroService;
import es.uva.tfg.geolocalizationgame.services.TipoLogroService;
import es.uva.tfg.geolocalizationgame.services.UserService;

/**
 * Controlador rest para la entidad Logro.
 * 
 * @author Cristina Gomez Escribano
 * @version 2.0
 * @since 15/12/2018
 *
 */
@RestController
@RequestMapping("/usuarios/{login}/logros")
public class LogroRestController {
	
	@Autowired
	private UserService userService;
	
	@Autowired
	private LogroService logroService;
	
	@Autowired
	private TipoLogroService tipoLogroService;

	/**
	 * Recibe la peticion para actualizar un logro en el sistema.
	 * 
	 * @param login
	 * @param idLogro
	 * @param visibilidad
	 * @return
	 * @throws UserNotFoundException
	 * @throws TipoLogroNotFoundException
	 * @throws LogroNotFoundException
	 * @throws TipoVisibilidadNotFoundException
	 * @throws MalformedRequestException
	 */
	@JsonView(DatosUser.Basic.class)
	@RequestMapping(value="{idLogro}", method=RequestMethod.PUT, produces=MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<Logro> putLogro(@PathVariable String login, @PathVariable Integer idLogro, @RequestParam Integer visibilidad) throws UserNotFoundException, TipoLogroNotFoundException, LogroNotFoundException, TipoVisibilidadNotFoundException, MalformedRequestException {
		Usuario usuario=userService.obtenerUsuarioByLogin(login);
		TipoLogro tipoLogro = tipoLogroService.obtenerTipoLogroById(idLogro);
		if(usuario==null) {
			throw new UserNotFoundException();
		}else if(tipoLogro==null){
			throw new TipoLogroNotFoundException();
		}else {
			Logro logro=logroService.obtenerLogroById(usuario, tipoLogro);
			if(logro==null) {
				throw new LogroNotFoundException();
			}else if (!logroService.actualizarVisibilidad(logro,visibilidad)){//no se ha encontrado el tipo de visibilidad
				throw new TipoVisibilidadNotFoundException();
			}else if(logroService.updateLogro(logro)!=null){//persistimos el cambio, todo ok devolvemos la respuesta
				return new ResponseEntity<>(logro,HttpStatus.ACCEPTED);
			}else {//error
				throw new MalformedRequestException();
			}
		}
	}
	
	/**
	 * Recibe la peticion para obtener los logros nuevos conseguidos por un usuario concreto.
	 * 
	 * @param login
	 * @return
	 * @throws UserNotFoundException
	 * @throws TipoVisibilidadNotFoundException
	 */
	@JsonView(DatosUser.Basic.class)
	@RequestMapping(value="logrosNuevos", method=RequestMethod.GET, produces=MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<List<Logro>> getLogrosNuevos(@PathVariable String login) throws UserNotFoundException, TipoVisibilidadNotFoundException {
		Usuario usuario=userService.obtenerUsuarioByLogin(login);
		if(usuario==null) {
			throw new UserNotFoundException();
		}
		
		return new ResponseEntity<List<Logro>>(logroService.obtenerLogrosNuevos(usuario), HttpStatus.OK);
	}
}
