package es.uva.tfg.geolocalizationgame.controllers;

import java.util.List;

import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.fasterxml.jackson.annotation.JsonView;

import es.uva.tfg.geolocalizationgame.exceptions.EstadoSolicitudNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.MalformedRequestException;
import es.uva.tfg.geolocalizationgame.exceptions.SolicitudAmistadExistException;
import es.uva.tfg.geolocalizationgame.exceptions.SolicitudAmistadNotFoundException;
import es.uva.tfg.geolocalizationgame.resources.DatosUser;
import es.uva.tfg.geolocalizationgame.resources.SolicitudAmistad;
import es.uva.tfg.geolocalizationgame.services.SolicitudAmistadService;

/**
 * Controlador rest que contiene las operaciones crud para la entidad de
 * Solicitud de Amistad
 * 
 * @author Cristina Gómez Escribano
 * @version 1.0
 * @since 29/08/2018
 *
 */
@RestController
@RequestMapping("/solicitudes")
public class SolicitudAmistadRestController {

	@Autowired
	public SolicitudAmistadService solicitudService;

	/**
	 * Recibe la petición post para crear una solicitud de amistad nueva.
	 * 
	 * @param solicitud
	 * @return
	 * @throws MalformedRequestException
	 * @throws SolicitudAmistadExistException
	 * @throws JSONException
	 */
	@RequestMapping(method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<Void> postSolicitudAmistad(@RequestBody String solicitud)
			throws MalformedRequestException, SolicitudAmistadExistException {
		try {
			JSONObject json = new JSONObject(solicitud);
			String emisor = json.getString("emisor");
			String receptor = json.getString("receptor");
			if (!solicitudService.exitsSolicitud(emisor, receptor)) {
				if (!solicitudService.createSolicitudAmistad(emisor, receptor)) {
					throw new MalformedRequestException("");
				}
			} else {//si ya existe, error
				throw new SolicitudAmistadExistException("");
			}
		} catch (JSONException e) {
			throw new MalformedRequestException("Json inválido");
		}
		
		return new ResponseEntity<>(HttpStatus.CREATED);
	}
	
	/**
	 * Recibe la petición para actualizar el estado de una solicitud de amistad por id
	 * @param idSolicitud
	 * @param estado
	 * @return ResponseEntity<Void>
	 * @throws MalformedRequestException
	 * @throws SolicitudAmistadNotFoundException 
	 * @throws EstadoSolicitudNotFoundException 
	 */
	@RequestMapping(value="{idSolicitud}", method=RequestMethod.PUT, produces=MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<Void> putSolicitudAmistad(@PathVariable Integer idSolicitud, @RequestParam String estado) throws MalformedRequestException, SolicitudAmistadNotFoundException, EstadoSolicitudNotFoundException{
		SolicitudAmistad solicitud = solicitudService.obtenerSolicitudById(idSolicitud);
		if(solicitud==null) {//no existe la solicitud
			throw new SolicitudAmistadNotFoundException();
		}
		else {
			if(solicitudService.actualizarEstado(solicitud, estado)) {//se actualiza el estado
				if(solicitudService.updateSolicitud(solicitud)!=null) {//se ha persistido el cambio
					return new ResponseEntity<>(HttpStatus.ACCEPTED);
				}else {
					throw new MalformedRequestException();
				}
			}else {//no se ha encontrado el estado de solicitud
				throw new EstadoSolicitudNotFoundException();
			}
		}
	}
	
	/**
	 * Recibe la peticion para obtenes las solicitudes enviadadas por un usuario.
	 * 
	 * @param login
	 * @param estado
	 * @return ResponseEntity<List<SolicitudAmistad>>
	 */
	@JsonView(DatosUser.WithSolicitudesEnviadas.class)
	@RequestMapping(value="{login}/solicitudesEnviadas", method=RequestMethod.GET, produces=MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<List<SolicitudAmistad>> getSolicitudesAmistadEnviadas(@PathVariable String login, @RequestParam String estado){
		return new ResponseEntity<>(solicitudService.obtenerSolicitudesEnviadasByEstado(login, estado), HttpStatus.OK);
	}
	
	/**
	 * Recibe la peticion para obtener las solicitudes recibidas por un usuario.
	 * 
	 * @param login
	 * @param estado
	 * @return ResponseEntity<List<SolicitudAmistad>>
	 */
	@JsonView(DatosUser.WithSolicitudesRecibidas.class)
	@RequestMapping(value="{login}/solicitudesRecibidas", method=RequestMethod.GET, produces=MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<List<SolicitudAmistad>> getSolicitudesAmistadRecibidas(@PathVariable String login, @RequestParam String estado){
		return new ResponseEntity<>(solicitudService.obtenerSolicitudesRecibidasByEstado(login, estado), HttpStatus.OK);
	}

}
