package es.uva.tfg.geolocalizationgame.controllers;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.TransactionSystemException;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.fasterxml.jackson.annotation.JsonView;

import es.uva.tfg.geolocalizationgame.exceptions.AmistadExistException;
import es.uva.tfg.geolocalizationgame.exceptions.AmistadNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.MalformedRequestException;
import es.uva.tfg.geolocalizationgame.exceptions.UserExistException;
import es.uva.tfg.geolocalizationgame.exceptions.UserNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.UserPassException;
import es.uva.tfg.geolocalizationgame.resources.DatosUser;
import es.uva.tfg.geolocalizationgame.resources.MapeoUser;
import es.uva.tfg.geolocalizationgame.resources.Usuario;
import es.uva.tfg.geolocalizationgame.services.UserService;

/**
 * Controlador que implementa el servicio REST con las operaciones CRUD para la
 * entidad usuario
 * 
 * @author Cristina Gómez Escribano
 * @version 1.0
 * @since 02/05/2018
 *
 */
@RestController
@RequestMapping("/usuarios")
public class UsuarioRestController {

	@Autowired
	private UserService userService;

	/**
	 * Recibe la peticion post para agregar un usuario en la base de datos.
	 * 
	 * @param usuario
	 * @return ResponseEntity<Void> con el status http correspondiente
	 * @throws MalformedRequestException
	 * @throws UserExistException
	 */
	@RequestMapping(value = "/registro", method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<Void> postUsuario(@RequestBody Usuario usuario)
			throws MalformedRequestException, UserExistException {
		try {
			if (userService.altaUsuario(usuario)) {
				return new ResponseEntity<>(HttpStatus.CREATED);
			} else { // el usuario ya existe
				throw new UserExistException("");
			}

		} catch (TransactionSystemException ex) { // se ha producido un error a la hora de persistir los datos
			throw new MalformedRequestException(ex.getMessage());
		}
	}

	/**
	 * Recibe la petición put para para actualizar los datos de usuario de la base
	 * de datos.
	 * 
	 * @param usuario
	 * @return
	 * @throws MalformedRequestException
	 * @throws UserPassException
	 */
	@JsonView(DatosUser.Basic.class)
	@RequestMapping(value = "{login}", method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<Usuario> putUsuario(@PathVariable String login, @RequestBody MapeoUser usuario)
			throws MalformedRequestException, UserPassException {
		try {
			Usuario userCurrent = userService.obtenerUsuarioByLogin(login);
			if (userCurrent == null) {// no encuentra el usuario
				throw new MalformedRequestException("");
			} else {
				if (!userService.actualizarDatosUsuario(usuario, userCurrent)) {// las pass no coinciden
					throw new UserPassException("");
				} else {
					Usuario newUser = userService.updateUserData(userCurrent);
					if (newUser != null) {// se ha actualizado correctamente
						return new ResponseEntity<>(newUser, HttpStatus.OK);
					} else {
						throw new MalformedRequestException("");
					}
				}
			}
		} catch (TransactionSystemException ex) {
			throw new MalformedRequestException(ex.getMessage());
		}
	}
	/**
	 * Recibe la peticion para actualizar los amigos de un usuario.
	 * 
	 * @param login
	 * @param loginAmigo
	 * @param agregarAmigo
	 * @return
	 * @throws MalformedRequestException
	 * @throws UserPassException
	 * @throws UserNotFoundException
	 * @throws AmistadExistException
	 * @throws AmistadNotFoundException
	 */
	@RequestMapping(value = "{login}/amigos/{loginAmigo}", method = RequestMethod.PUT, produces = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<Void> putAmigosUsuario(@PathVariable String login, @PathVariable String loginAmigo, @RequestParam boolean agregarAmigo)
			throws MalformedRequestException, UserPassException, UserNotFoundException, AmistadExistException, AmistadNotFoundException {
		try {
			Usuario usuario = userService.obtenerUsuarioByLogin(login);
			Usuario newAmigo = userService.obtenerUsuarioByLogin(loginAmigo);
			if (usuario == null) {// no se ha encontrado el usuario
				throw new UserNotFoundException("No se ha encontrado el usuario");
			} else if (newAmigo == null) {
				throw new UserNotFoundException("No se ha encontrado el nuevo amigo");
			} 
			
			if(agregarAmigo) {//voy a agregar un nuevo amigo
				if(!userService.agregarAmigo(usuario, newAmigo)) {//ya existe la relacion de amistad
					throw new AmistadExistException();
				}
			}else {//se va a eliminar un amigo
				if(!userService.eliminarAmigo(usuario, newAmigo)) {//no se ha encontrado la relacion de amistad
					throw new AmistadNotFoundException();
				}
			}
				
			if (userService.updateUserData(usuario) != null) {// se ha actualizado correctamente
				return new ResponseEntity<>(HttpStatus.OK);
			} else {
				throw new MalformedRequestException();
			}
			
		} catch (TransactionSystemException ex) {
			throw new MalformedRequestException(ex.getMessage());
		}
	}

	/**
	 * Recibe la petición get para obtener la lista de usuarios. Puede incluir un
	 * parametro para buscar usuarios por texto.
	 * 
	 * @param text
	 *            (no required)
	 * @return ResponseEntity<List<Usuario>>
	 * @throws MalformedRequestException
	 */
	@JsonView(DatosUser.Basic.class)
	@RequestMapping(method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<List<Usuario>> getUsuarios(@RequestParam(required = false) String text)
			throws MalformedRequestException {
		try {
			List<Usuario> usuarios = userService.getUsuarios(text);
			return new ResponseEntity<>(usuarios, HttpStatus.OK);
		} catch (Exception ex) {
			throw new MalformedRequestException("");
		}
	}

	/**
	 * Recibe la petición get para obtener un usuario por login
	 * 
	 * @return ResponseEntity<Usuario>
	 * @throws MalformedRequestException
	 */
	@JsonView(DatosUser.PrivateProfile.class)
	@RequestMapping(value = "/{login}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<Usuario> getUsuario(@PathVariable String login) throws MalformedRequestException {
		try {
			Usuario usuario = userService.obtenerUsuarioByLogin(login);
			if (usuario == null) {// no encuentra el usuario
				throw new MalformedRequestException("");
			} else {
				userService.actualizarLogrosUsuario(usuario);//dejar solo los logros publicos que son los visibles
				userService.actualizarJuegosUsuario(usuario);
				return new ResponseEntity<>(usuario, HttpStatus.OK);
			}
		} catch (Exception ex) {
			throw new MalformedRequestException("");
		}
	}
	
	/**
	 * Recibe la peticion para obtener los amigos de un usuario
	 * 
	 * @return ResponseEntity<List<Usuario>>
	 * @throws MalformedRequestException
	 * @throws 
	 */
	@JsonView(DatosUser.PrivateProfile.class)
	@RequestMapping(value = "/{login}/amigos", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<List<Usuario>> getAmigos(@PathVariable String login) throws MalformedRequestException, UserNotFoundException {
		try {
			Usuario usuario = userService.obtenerUsuarioByLogin(login);
			if (usuario == null) {// no encuentra el usuario
				throw new UserNotFoundException();
			} else {
				userService.actualizarLogrosAmigos(usuario);
				userService.actualizarJuegosAmigos(usuario);
				return new ResponseEntity<>(usuario.getAmigos(), HttpStatus.OK);
			}
		} catch (Exception ex) {
			throw new MalformedRequestException("");
		}
	}

}
