package es.uva.tfg.geolocalizationgame.services;

import java.util.Date;
import java.util.List;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import es.uva.tfg.geolocalizationgame.exceptions.TipoVisibilidadNotFoundException;
import es.uva.tfg.geolocalizationgame.persistence.JuegoRepository;
import es.uva.tfg.geolocalizationgame.resources.Juego;
import es.uva.tfg.geolocalizationgame.resources.TipoVisibilidad;
import es.uva.tfg.geolocalizationgame.resources.Usuario;


/**
 * Implementa la clase de servicio para la entidad Juego.
 * 
 * @author Cristina Gomez Escribano
 * @since 10/11/2018
 * @version 2.0
 */
@Service
public class JuegoServiceImpl implements JuegoService {

	@Autowired
	private JuegoRepository juegoRepository;
	
	@Autowired
	private TipoVisibilidadService tipoVisibilidadService;
	
	@Override
	public Juego obtenerJuegoById(Integer idJuego) {
		return juegoRepository.findOne(idJuego);
	}

	@Override
	public void eliminarJuego(@Valid Juego juego) {
		juegoRepository.deleteAciertos(juego.getIdJuego());
		juegoRepository.deleteFallos(juego.getIdJuego());
		juegoRepository.deleteConseguidos(juego.getIdJuego());
		juegoRepository.delete(juego);
	}

	@Override
	public List<Juego> obtenerJuegosByLogin(@Valid Usuario usuario) {
		return juegoRepository.obtenerJuegosUser(usuario);
	}

	@Override
	public List<Juego> obtenerJuegosPublicosByText(String texto) {
		return juegoRepository.obtenerJuegosPublicosByTexto(texto.toUpperCase());
	}

	@Override
	public boolean compruebaJuegoPermitido(@Valid Usuario usuario, @Valid Juego juego) {
		boolean permitido=false;
		if(juego.getCreador().equals(usuario)) {//si es el creado lo devuelves
			permitido=true;
		}else {
			if(juego.getVisibilidad().getTipoVisibilidad().equalsIgnoreCase("publico")) {
				permitido=true;
			}else if(juego.getVisibilidad().getTipoVisibilidad().equalsIgnoreCase("amigos") && comprobarAmigo(usuario, juego)) {//tiene  visibilidad amigo y es amigo del usuario
				permitido=true;
			}
		}
		
		return permitido;
	}

	/**
	 * Devuelve si el creador de un juego es amigo del usuario autenticado
	 * 
	 * @param usuario
	 * @param juego
	 * @return
	 */
	private boolean comprobarAmigo(@Valid Usuario usuario, @Valid Juego juego) {
		boolean amigo=false;
		Usuario creador=juego.getCreador();
		List<Usuario> amigos=usuario.getAmigos();
		for(int i=0; i<amigos.size();i++) {
			if(amigos.get(i).equals(creador)) {//si el creador es amigo del usuario
				amigo=true;
				break;
			}
		}
		return amigo;
	}

	@Override
	public void actualizarNombre(@Valid Juego juego, String nombre) {
		juego.setNombre(nombre);
	}

	@Override
	public Juego updateJuego(@Valid Juego juego) {
		return juegoRepository.saveAndFlush(juego);
	}

	@Override
	public Juego crearJuego(@Valid Usuario usuario, String nombreJuego) throws TipoVisibilidadNotFoundException {
		TipoVisibilidad visibilidad=tipoVisibilidadService.obtenerVisibilidadByNombre("privado");
		if(visibilidad==null) {
			throw new TipoVisibilidadNotFoundException();
		}
		Juego juego = new Juego();
		juego.setNombre(nombreJuego);
		juego.setCreador(usuario);
		juego.setFechaCreacion(new Date());
		juego.setFechaModificacion(new Date());
		juego.setVisibilidad(visibilidad);
		return juego;
	}

	@Override
	public void saveJuego(@Valid Juego juegoCreado) {
		juegoRepository.saveAndFlush(juegoCreado);
	}

	@Override
	public boolean actualizarVisibilidad(Juego juego, Integer visibilidad) {
		boolean update=false;
		TipoVisibilidad tipoVisibilidad=tipoVisibilidadService.obtenerVisibilidadById(visibilidad);
		if(tipoVisibilidad!=null) {//se ha encontrado, por tanto se actualiza
			juego.setVisibilidad(tipoVisibilidad);
			juego.setFechaModificacion(new Date());
			update=true;
		}
		return update;
		
	}
}